"use strict";
// Copyright IBM Corp. 2019. All Rights Reserved.
// Node module: @loopback/context
// This file is licensed under the MIT License.
// License text available at https://opensource.org/licenses/MIT
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Type guard for binding address
 * @param bindingSelector
 */
function isBindingAddress(bindingSelector) {
    return typeof bindingSelector !== 'function';
}
exports.isBindingAddress = isBindingAddress;
/**
 * Type guard for BindingTagFilter
 * @param filter - A BindingFilter function
 */
function isBindingTagFilter(filter) {
    if (filter == null || !('bindingTagPattern' in filter))
        return false;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const tagPattern = filter.bindingTagPattern;
    return (tagPattern instanceof RegExp ||
        typeof tagPattern === 'string' ||
        typeof tagPattern === 'object');
}
exports.isBindingTagFilter = isBindingTagFilter;
/**
 * Create a binding filter for the tag pattern
 * @param tagPattern - Binding tag name, regexp, or object
 */
function filterByTag(tagPattern) {
    let filter;
    let regex = undefined;
    if (tagPattern instanceof RegExp) {
        // RegExp for tag names
        regex = tagPattern;
    }
    if (typeof tagPattern === 'string' &&
        (tagPattern.includes('*') || tagPattern.includes('?'))) {
        // Wildcard tag name
        regex = wildcardToRegExp(tagPattern);
    }
    if (regex != null) {
        // RegExp or wildcard match
        filter = b => b.tagNames.some(t => regex.test(t));
    }
    else if (typeof tagPattern === 'string') {
        // Plain tag string match
        filter = b => b.tagNames.includes(tagPattern);
    }
    else {
        // Match tag name/value pairs
        const tagMap = tagPattern;
        filter = b => {
            for (const t in tagPattern) {
                // One tag name/value does not match
                if (b.tagMap[t] !== tagMap[t])
                    return false;
            }
            // All tag name/value pairs match
            return true;
        };
    }
    // Set up binding tag for the filter
    const tagFilter = filter;
    tagFilter.bindingTagPattern = (regex !== null && regex !== void 0 ? regex : tagPattern);
    return tagFilter;
}
exports.filterByTag = filterByTag;
/**
 * Create a binding filter from key pattern
 * @param keyPattern - Binding key/wildcard, regexp, or a filter function
 */
function filterByKey(keyPattern) {
    if (typeof keyPattern === 'string') {
        const regex = wildcardToRegExp(keyPattern);
        return binding => regex.test(binding.key);
    }
    else if (keyPattern instanceof RegExp) {
        return binding => keyPattern.test(binding.key);
    }
    else if (typeof keyPattern === 'function') {
        return keyPattern;
    }
    return () => true;
}
exports.filterByKey = filterByKey;
/**
 * Convert a wildcard pattern to RegExp
 * @param pattern - A wildcard string with `*` and `?` as special characters.
 * - `*` matches zero or more characters except `.` and `:`
 * - `?` matches exactly one character except `.` and `:`
 */
function wildcardToRegExp(pattern) {
    // Escape reserved chars for RegExp:
    // `- \ ^ $ + . ( ) | { } [ ] :`
    let regexp = pattern.replace(/[\-\[\]\/\{\}\(\)\+\.\\\^\$\|\:]/g, '\\$&');
    // Replace wildcard chars `*` and `?`
    // `*` matches zero or more characters except `.` and `:`
    // `?` matches one character except `.` and `:`
    regexp = regexp.replace(/\*/g, '[^.:]*').replace(/\?/g, '[^.:]');
    return new RegExp(`^${regexp}$`);
}
//# sourceMappingURL=binding-filter.js.map